@@ -16,6 +16,8 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
+#define G_LOG_DOMAIN "GtdProviderEds"
+
 #include "gtd-provider-eds.h"
 #include "gtd-task-eds.h"
 #include "gtd-task-list-eds.h"
@@ -53,10 +55,21 @@ typedef struct _TaskData
 } TaskData;
 
 
-G_DEFINE_TYPE_WITH_PRIVATE (GtdProviderEds, gtd_provider_eds, GTD_TYPE_OBJECT)
+static void          gtd_provider_iface_init                     (GtdProviderInterface *iface);
 
-enum {
+G_DEFINE_TYPE_WITH_CODE (GtdProviderEds, gtd_provider_eds, GTD_TYPE_OBJECT,
+                         G_ADD_PRIVATE (GtdProviderEds)
+                         G_IMPLEMENT_INTERFACE (GTD_TYPE_PROVIDER, gtd_provider_iface_init))
+
+enum
+{
   PROP_0,
+  PROP_ENABLED,
+  PROP_DEFAULT_TASKLIST,
+  PROP_DESCRIPTION,
+  PROP_ICON,
+  PROP_ID,
+  PROP_NAME,
   PROP_REGISTRY,
   N_PROPS
 };
@@ -112,14 +125,17 @@ gtd_provider_eds_fill_task_list (GObject      *client,
                                  GAsyncResult *result,
                                  gpointer      user_data)
 {
+  g_autoptr (GError) error = NULL;
+  GtdProviderEds *self;
   GtdTaskList *list;
   TaskData *data = user_data;
   GSList *component_list;
-  GError *error = NULL;
+  GSList *l;
 
   g_return_if_fail (GTD_IS_PROVIDER_EDS (data->provider));
 
   list = GTD_TASK_LIST (data->data);
+  self = data->provider;
 
   e_cal_client_get_object_list_as_comps_finish (E_CAL_CLIENT (client),
                                                 result,
@@ -129,37 +145,32 @@ gtd_provider_eds_fill_task_list (GObject      *client,
   gtd_object_set_ready (GTD_OBJECT (data->data), TRUE);
   g_free (data);
 
-  if (!error)
+  if (error)
     {
-      GSList *l;
-
-      for (l = component_list; l != NULL; l = l->next)
-        {
-          GtdTask *task;
-
-          task = gtd_task_eds_new (l->data);
-          gtd_task_set_list (task, list);
-
-          gtd_task_list_save_task (list, task);
-        }
-
-      e_cal_client_free_ecalcomp_slist (component_list);
-    }
-  else
-    {
-      g_warning ("%s: %s: %s",
-                 G_STRFUNC,
-                 _("Error fetching tasks from list"),
-                 error->message);
+      g_warning ("Error fetching tasks from list: %s", error->message);
 
       gtd_manager_emit_error_message (gtd_manager_get_default (),
                                       _("Error fetching tasks from list"),
                                       error->message,
                                       NULL,
                                       NULL);
-      g_error_free (error);
       return;
     }
+
+  for (l = component_list; l != NULL; l = l->next)
+    {
+      GtdTask *task;
+
+      task = gtd_task_eds_new (l->data);
+      gtd_task_set_list (task, list);
+
+      gtd_task_list_save_task (list, task);
+    }
+
+  e_cal_client_free_ecalcomp_slist (component_list);
+
+  /* Emit LIST_ADDED signal */
+  g_signal_emit_by_name (self, "list-added", list);
 }
 
 static void
@@ -167,6 +178,7 @@ gtd_provider_eds_on_client_connected (GObject      *source_object,
                                       GAsyncResult *result,
                                       gpointer      user_data)
 {
+  g_autoptr (GError) error = NULL;
   GtdProviderEdsPrivate *priv;
   GtdProviderEds *self;
   GtdTaskListEds *list;
@@ -175,29 +187,21 @@ gtd_provider_eds_on_client_connected (GObject      *source_object,
   ESource *default_source;
   ESource *source;
   ESource *parent;
-  GError *error;
 
   self = GTD_PROVIDER_EDS (user_data);
   priv = gtd_provider_eds_get_instance_private (self);
-  error = NULL;
   source = e_client_get_source (E_CLIENT (source_object));
   client = E_CAL_CLIENT (e_cal_client_connect_finish (result, &error));
 
   if (error)
     {
-      g_debug ("%s: %s (%s): %s",
-               G_STRFUNC,
-               _("Failed to connect to task list"),
-               e_source_get_uid (source),
-               error->message);
+      g_debug ("Failed to connect to task list '%s': %s", e_source_get_uid (source), error->message);
 
       gtd_manager_emit_error_message (gtd_manager_get_default (),
                                       _("Failed to connect to task list"),
                                       error->message,
                                       NULL,
                                       NULL);
-
-      g_error_free (error);
       return;
     }
 
@@ -236,15 +240,9 @@ gtd_provider_eds_on_client_connected (GObject      *source_object,
 
   g_clear_object (&default_source);
 
-  /* Emit LIST_ADDED signal */
-  g_signal_emit_by_name (self, "list-added", list);
-
   g_object_unref (parent);
 
-  g_debug ("%s: %s (%s)",
-           G_STRFUNC,
-           _("Task list source successfully connected"),
-           e_source_get_display_name (source));
+  g_debug ("Task list '%s' successfully connected", e_source_get_display_name (source));
 }
 
 typedef struct
@@ -336,10 +334,12 @@ gtd_manager__invoke_authentication (GObject      *source_object,
                                     GAsyncResult *result,
                                     gpointer      user_data)
 {
-  ESource *source = E_SOURCE (source_object);
-  GError *error = NULL;
+  g_autoptr (GError) error = NULL;
+  ESource *source;
   gboolean canceled;
 
+  source = E_SOURCE (source_object);
+
   e_source_invoke_authenticate_finish (source,
                                        result,
                                        &error);
@@ -349,15 +349,7 @@ gtd_manager__invoke_authentication (GObject      *source_object,
                                    G_IO_ERROR_CANCELLED);
 
   if (!canceled)
-    {
-      g_warning ("%s: %s (%s): %s",
-                 G_STRFUNC,
-                 _("Failed to prompt for credentials"),
-                 e_source_get_uid (source),
-                 error->message);
-    }
-
-  g_clear_error (&error);
+    g_warning ("Failed to prompt for credentials (%s): %s", e_source_get_uid (source), error->message);
 }
 
 static void
@@ -367,7 +359,7 @@ gtd_provider_local_credentials_prompt_done (GObject      *source_object,
 {
   ETrustPromptResponse response = E_TRUST_PROMPT_RESPONSE_UNKNOWN;
   ESource *source = E_SOURCE (source_object);
-  GError *error = NULL;
+  g_autoptr (GError) error = NULL;
 
   e_trust_prompt_run_for_source_finish (source, result, &response, &error);
 
@@ -375,22 +367,21 @@ gtd_provider_local_credentials_prompt_done (GObject      *source_object,
     {
       g_warning ("%s: %s '%s': %s",
                  G_STRFUNC,
-                 _("Failed to prompt for credentials for"),
+                 "Failed to prompt for credentials for",
                  e_source_get_display_name (source),
                  error->message);
-
-    }
-  else if (response == E_TRUST_PROMPT_RESPONSE_ACCEPT || response == E_TRUST_PROMPT_RESPONSE_ACCEPT_TEMPORARILY)
-    {
-      /* Use NULL credentials to reuse those from the last time. */
-      e_source_invoke_authenticate (source,
-                                    NULL,
-                                    NULL /* cancellable */,
-                                    gtd_manager__invoke_authentication,
-                                    NULL);
+      return;
     }
 
-  g_clear_error (&error);
+  if (response != E_TRUST_PROMPT_RESPONSE_ACCEPT && response != E_TRUST_PROMPT_RESPONSE_ACCEPT_TEMPORARILY)
+      return;
+
+  /* Use NULL credentials to reuse those from the last time. */
+  e_source_invoke_authenticate (source,
+                                NULL,
+                                NULL /* cancellable */,
+                                gtd_manager__invoke_authentication,
+                                NULL);
 }
 
 static void
@@ -427,9 +418,7 @@ gtd_provider_eds_credentials_required (ESourceRegistry          *registry,
     }
   else if (error && reason == E_SOURCE_CREDENTIALS_REASON_ERROR)
     {
-      g_warning ("%s: %s '%s': %s",
-                 G_STRFUNC,
-                 _("Authentication failure"),
+      g_warning ("Authentication failure '%s': %s",
                  e_source_get_display_name (source),
                  error->message);
     }
@@ -456,55 +445,19 @@ out:
   g_clear_object (&default_source);
 }
 
-static void
-gtd_provider_eds_finalize (GObject *object)
-{
-  GtdProviderEds *self = (GtdProviderEds *)object;
-  GtdProviderEdsPrivate *priv = gtd_provider_eds_get_instance_private (self);
-
-  g_signal_handlers_disconnect_by_func (priv->source_registry, default_tasklist_changed_cb, self);
-
-  g_clear_pointer (&priv->clients, g_hash_table_destroy);
-  g_clear_object (&priv->credentials_prompter);
-  g_clear_object (&priv->source_registry);
-
-  G_OBJECT_CLASS (gtd_provider_eds_parent_class)->finalize (object);
-}
-
-static void
-gtd_provider_eds_get_property (GObject    *object,
-                               guint       prop_id,
-                               GValue     *value,
-                               GParamSpec *pspec)
-{
-  GtdProviderEds *self = GTD_PROVIDER_EDS (object);
-  GtdProviderEdsPrivate *priv = gtd_provider_eds_get_instance_private (self);
-
-  switch (prop_id)
-    {
-    case PROP_REGISTRY:
-      g_value_set_object (value, priv->source_registry);
-      break;
-
-    default:
-      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
-    }
-}
-
 static void
 gtd_provider_eds_load_registry (GtdProviderEds  *provider)
 {
   GtdProviderEdsPrivate *priv = gtd_provider_eds_get_instance_private (provider);
+  g_autoptr (GError) error = NULL;
   GList *sources;
   GList *l;
-  GError *error = NULL;
 
   priv->credentials_prompter = e_credentials_prompter_new (priv->source_registry);
 
-  if (error != NULL)
+  if (error)
     {
-      g_warning ("%s: %s", _("Error loading task manager"), error->message);
-      g_error_free (error);
+      g_warning ("%s: %s", "Error loading task manager", error->message);
       return;
     }
 
@@ -567,244 +520,404 @@ gtd_provider_eds_set_registry (GtdProviderEds  *provider,
 }
 
 
-static void
-gtd_provider_eds_create_task_finished (GObject      *client,
-                                       GAsyncResult *result,
-                                       gpointer      user_data)
+/*
+ * GtdProvider iface
+ */
+
+static const gchar*
+gtd_provider_eds_get_id (GtdProvider *provider)
 {
-  TaskData *data = user_data;
-  gchar *new_uid = NULL;
-  GError *error = NULL;
+  g_return_val_if_fail (GTD_IS_PROVIDER_EDS (provider), NULL);
 
-  e_cal_client_create_object_finish (E_CAL_CLIENT (client),
-                                     result,
-                                     &new_uid,
-                                     &error);
+  return GTD_PROVIDER_EDS_CLASS (G_OBJECT_GET_CLASS (provider))->get_id (GTD_PROVIDER_EDS (provider));
+}
 
-  gtd_object_set_ready (GTD_OBJECT (data->data), TRUE);
+static const gchar*
+gtd_provider_eds_get_name (GtdProvider *provider)
+{
+  g_return_val_if_fail (GTD_IS_PROVIDER_EDS (provider), NULL);
 
-  if (error)
+  return GTD_PROVIDER_EDS_CLASS (G_OBJECT_GET_CLASS (provider))->get_name (GTD_PROVIDER_EDS (provider));
+}
+
+static const gchar*
+gtd_provider_eds_get_description (GtdProvider *provider)
+{
+  g_return_val_if_fail (GTD_IS_PROVIDER_EDS (provider), NULL);
+
+  return GTD_PROVIDER_EDS_CLASS (G_OBJECT_GET_CLASS (provider))->get_description (GTD_PROVIDER_EDS (provider));
+}
+
+
+static gboolean
+gtd_provider_eds_get_enabled (GtdProvider *provider)
+{
+  g_return_val_if_fail (GTD_IS_PROVIDER_EDS (provider), FALSE);
+
+  return GTD_PROVIDER_EDS_CLASS (G_OBJECT_GET_CLASS (provider))->get_enabled (GTD_PROVIDER_EDS (provider));
+}
+
+static GIcon*
+gtd_provider_eds_get_icon (GtdProvider *provider)
+{
+  g_return_val_if_fail (GTD_IS_PROVIDER_EDS (provider), NULL);
+
+  return GTD_PROVIDER_EDS_CLASS (G_OBJECT_GET_CLASS (provider))->get_icon (GTD_PROVIDER_EDS (provider));
+}
+
+static void
+gtd_provider_eds_create_task (GtdProvider   *provider,
+                              GtdTask       *task,
+                              GCancellable  *cancellable,
+                              GError       **error)
+{
+  g_autofree gchar *new_uid = NULL;
+  GtdProviderEdsPrivate *priv;
+  GtdProviderEds *self;
+  GtdTaskListEds *tasklist;
+  ECalComponent *component;
+  ECalClient *client;
+  ESource *source;
+
+  g_return_if_fail (GTD_IS_TASK (task));
+  g_return_if_fail (GTD_IS_TASK_LIST_EDS (gtd_task_get_list (task)));
+
+  self = GTD_PROVIDER_EDS (provider);
+  priv = gtd_provider_eds_get_instance_private (self);
+  tasklist = GTD_TASK_LIST_EDS (gtd_task_get_list (task));
+  source = gtd_task_list_eds_get_source (tasklist);
+  client = g_hash_table_lookup (priv->clients, source);
+  component = gtd_task_eds_get_component (GTD_TASK_EDS (task));
+
+  /* The task is not ready until we finish the operation */
+  gtd_object_set_ready (GTD_OBJECT (task), FALSE);
+
+  if (!e_cal_client_create_object_sync (client,
+                                        e_cal_component_get_icalcomponent (component),
+                                        &new_uid,
+                                        cancellable,
+                                        error))
     {
-      GtdTask *task = GTD_TASK (data->data);
-
-      g_warning ("%s: %s: %s",
-                 G_STRFUNC,
-                 _("Error creating task"),
-                 error->message);
-
-      /* Remove from the tasklist */
       gtd_task_list_remove_task (gtd_task_get_list (task), task);
-
-      /* Display a notification */
-      gtd_manager_emit_error_message (gtd_manager_get_default (),
-                                      _("Error creating task"),
-                                      error->message,
-                                      NULL,
-                                      NULL);
-
-      g_error_free (error);
     }
   else
     {
       /* Update the default tasklist */
-      gtd_provider_eds_set_default (data->provider, gtd_task_get_list (GTD_TASK (data->data)));
+      gtd_provider_eds_set_default (self, gtd_task_get_list (task));
 
       /*
        * In the case the task UID changes because of creation proccess,
        * reapply it to the task.
        */
       if (new_uid)
-        {
-          gtd_object_set_uid (GTD_OBJECT (data->data), new_uid);
-          g_free (new_uid);
-        }
-    }
-
-  g_free (data);
-}
-
-static void
-gtd_provider_eds_update_task_finished (GObject      *client,
-                                       GAsyncResult *result,
-                                       gpointer      user_data)
-{
-  TaskData *data = user_data;
-  GError *error = NULL;
-
-  e_cal_client_modify_object_finish (E_CAL_CLIENT (client),
-                                     result,
-                                     &error);
-
-  if (error)
-    {
-      g_warning ("%s: %s: %s",
-                 G_STRFUNC,
-                 _("Error updating task"),
-                 error->message);
-
-      gtd_manager_emit_error_message (gtd_manager_get_default (),
-                                      _("Error updating task"),
-                                      error->message,
-                                      NULL,
-                                      NULL);
-
-      g_error_free (error);
-    }
-
-  gtd_object_set_ready (GTD_OBJECT (data->data), TRUE);
-
-  g_free (data);
-}
-
-static void
-gtd_provider_eds_remove_task_finished (GObject      *client,
-                                       GAsyncResult *result,
-                                       gpointer      user_data)
-{
-  TaskData *data = user_data;
-  GError *error = NULL;
-
-  e_cal_client_remove_object_finish (E_CAL_CLIENT (client),
-                                     result,
-                                     &error);
-
-  gtd_object_set_ready (GTD_OBJECT (data->data), TRUE);
-
-  g_object_unref ((GtdTask*) data->data);
-  g_free (data);
-
-  if (error)
-    {
-      g_warning ("%s: %s: %s",
-                 G_STRFUNC,
-                 _("Error removing task"),
-                 error->message);
-
-      gtd_manager_emit_error_message (gtd_manager_get_default (),
-                                      _("Error removing task"),
-                                      error->message,
-                                      NULL,
-                                      NULL);
-      g_error_free (error);
-      return;
+        gtd_object_set_uid (GTD_OBJECT (task), new_uid);
     }
 }
 
 static void
-gtd_provider_eds_remote_create_source_finished (GObject      *source,
-                                                GAsyncResult *result,
-                                                gpointer      user_data)
+gtd_provider_eds_update_task (GtdProvider   *provider,
+                              GtdTask       *task,
+                              GCancellable  *cancellable,
+                              GError       **error)
 {
-  GError *error;
+  GtdProviderEdsPrivate *priv;
+  GtdProviderEds *self;
+  GtdTaskListEds *tasklist;
+  ECalComponent *component;
+  ECalClient *client;
+  ESource *source;
 
-  error = NULL;
+  g_return_if_fail (GTD_IS_TASK (task));
+  g_return_if_fail (GTD_IS_TASK_LIST_EDS (gtd_task_get_list (task)));
 
-  e_source_remote_create_finish (E_SOURCE (source),
-                                 result,
-                                 &error);
+  self = GTD_PROVIDER_EDS (provider);
+  priv = gtd_provider_eds_get_instance_private (self);
+  tasklist = GTD_TASK_LIST_EDS (gtd_task_get_list (task));
+  source = gtd_task_list_eds_get_source (tasklist);
+  client = g_hash_table_lookup (priv->clients, source);
+  component = gtd_task_eds_get_component (GTD_TASK_EDS (task));
 
-  if (error)
-    {
-      g_warning ("%s: %s: %s",
-                 G_STRFUNC,
-                 _("Error creating task list"),
-                 error->message);
+  e_cal_component_commit_sequence (component);
 
-      gtd_manager_emit_error_message (gtd_manager_get_default (),
-                                      _("Error creating task list"),
-                                      error->message,
-                                      NULL,
-                                      NULL);
-      g_clear_error (&error);
-    }
+  /* The task is not ready until we finish the operation */
+  gtd_object_set_ready (GTD_OBJECT (task), FALSE);
+
+  e_cal_client_modify_object_sync (client,
+                                   e_cal_component_get_icalcomponent (component),
+                                   E_CAL_OBJ_MOD_THIS,
+                                   cancellable,
+                                   error);
+
+  gtd_object_set_ready (GTD_OBJECT (task), TRUE);
 }
 
 static void
-task_list_removal_finished (GtdProvider *provider,
-                            ESource     *source,
-                            GError     **error)
+gtd_provider_eds_remove_task (GtdProvider   *provider,
+                              GtdTask       *task,
+                              GCancellable  *cancellable,
+                              GError       **error)
 {
+
+  GtdProviderEdsPrivate *priv;
+  ECalComponentId *id;
+  GtdProviderEds *self;
+  GtdTaskListEds *tasklist;
+  ECalComponent *component;
+  ECalClient *client;
+  ESource *source;
+
+  g_return_if_fail (GTD_IS_TASK (task));
+  g_return_if_fail (GTD_IS_TASK_LIST_EDS (gtd_task_get_list (task)));
+
+  self = GTD_PROVIDER_EDS (provider);
+  priv = gtd_provider_eds_get_instance_private (self);
+  tasklist = GTD_TASK_LIST_EDS (gtd_task_get_list (task));
+  source = gtd_task_list_eds_get_source (tasklist);
+  client = g_hash_table_lookup (priv->clients, source);
+  component = gtd_task_eds_get_component (GTD_TASK_EDS (task));
+  id = e_cal_component_get_id (component);
+
+  /* The task is not ready until we finish the operation */
+  gtd_object_set_ready (GTD_OBJECT (task), FALSE);
+
+  e_cal_client_remove_object_sync (client,
+                                   id->uid,
+                                   id->rid,
+                                   E_CAL_OBJ_MOD_THIS,
+                                   cancellable,
+                                   error);
+
+  gtd_object_set_ready (GTD_OBJECT (task), TRUE);
+
+  e_cal_component_free_id (id);
+}
+
+static void
+gtd_provider_eds_create_task_list (GtdProvider   *provider,
+                                   GtdTaskList   *list,
+                                   GCancellable  *cancellable,
+                                   GError       **error)
+{
+  GtdProviderEdsPrivate *priv;
+  GtdProviderEds *self;
+  ESource *source;
+
+  self = GTD_PROVIDER_EDS (provider);
+  priv = gtd_provider_eds_get_instance_private (self);
+  source = NULL;
+
+  /* Create an ESource */
+  if (GTD_PROVIDER_EDS_CLASS (G_OBJECT_GET_CLASS (provider))->create_source)
+    source = GTD_PROVIDER_EDS_CLASS (G_OBJECT_GET_CLASS (provider))->create_source (self);
+
+  if (!source)
+    return;
+
+  /* EDS properties */
+  e_source_set_display_name (source, gtd_task_list_get_name (list));
+
+  gtd_object_set_ready (GTD_OBJECT (provider), FALSE);
+
+  e_source_registry_commit_source_sync (priv->source_registry,
+                                        source,
+                                        cancellable,
+                                        error);
+
+  gtd_object_set_ready (GTD_OBJECT (list), TRUE);
+}
+
+static void
+gtd_provider_eds_update_task_list (GtdProvider   *provider,
+                                   GtdTaskList   *list,
+                                   GCancellable  *cancellable,
+                                   GError       **error)
+{
+  GtdProviderEdsPrivate *priv;
+  ESource *source;
+
+  g_return_if_fail (GTD_IS_TASK_LIST (list));
+  g_return_if_fail (gtd_task_list_eds_get_source (GTD_TASK_LIST_EDS (list)));
+
+  priv = gtd_provider_eds_get_instance_private (GTD_PROVIDER_EDS (provider));
+  source = gtd_task_list_eds_get_source (GTD_TASK_LIST_EDS (list));
+
+  gtd_object_set_ready (GTD_OBJECT (provider), FALSE);
+  e_source_registry_commit_source_sync (priv->source_registry,
+                                        source,
+                                        cancellable,
+                                        error);
+
   gtd_object_set_ready (GTD_OBJECT (provider), TRUE);
 
-  g_object_notify (G_OBJECT (provider), "default-task-list");
+  g_signal_emit_by_name (provider, "list-changed", list);
+}
 
-  if (*error)
+static void
+gtd_provider_eds_remove_task_list (GtdProvider   *provider,
+                                   GtdTaskList   *list,
+                                   GCancellable  *cancellable,
+                                   GError       **error)
+{
+  ESource *source;
+
+  g_return_if_fail (GTD_IS_TASK_LIST (list));
+  g_return_if_fail (gtd_task_list_eds_get_source (GTD_TASK_LIST_EDS (list)));
+
+  error = NULL;
+  source = gtd_task_list_eds_get_source (GTD_TASK_LIST_EDS (list));
+
+  gtd_object_set_ready (GTD_OBJECT (provider), FALSE);
+
+  e_source_remove_sync (source, cancellable, error);
+
+  gtd_object_set_ready (GTD_OBJECT (provider), TRUE);
+
+  g_signal_emit_by_name (provider, "list-removed", list);
+}
+
+static GList*
+gtd_provider_eds_get_task_lists (GtdProvider *provider)
+{
+  GtdProviderEdsPrivate *priv = gtd_provider_eds_get_instance_private (GTD_PROVIDER_EDS (provider));
+
+  return priv->task_lists;
+}
+
+static GtdTaskList*
+gtd_provider_eds_get_default_task_list (GtdProvider *provider)
+{
+  GtdProviderEdsPrivate *priv;
+  GtdTaskList *default_task_list;
+  ESource *default_source;
+
+  priv = gtd_provider_eds_get_instance_private (GTD_PROVIDER_EDS (provider));
+  default_source = e_source_registry_ref_default_task_list (priv->source_registry);
+  default_task_list = g_object_get_data (G_OBJECT (default_source), "task-list");
+
+  g_clear_object (&default_source);
+
+  if (default_task_list &&
+      gtd_task_list_get_provider (default_task_list) != GTD_PROVIDER (provider))
     {
-      g_warning ("%s: %s: %s",
-                 G_STRFUNC,
-                 _("Error removing task list"),
-                 (*error)->message);
-
-      gtd_manager_emit_error_message (gtd_manager_get_default (),
-                                      _("Error removing task list"),
-                                      (*error)->message,
-                                      NULL,
-                                      NULL);
-      g_clear_error (error);
+      return NULL;
     }
-}
 
-
-static void
-gtd_provider_eds_remote_delete_finished (GObject      *source,
-                                         GAsyncResult *result,
-                                         gpointer      user_data)
-{
-  GError *error = NULL;
-
-  e_source_remote_delete_finish (E_SOURCE (source),
-                                 result,
-                                 &error);
-
-  task_list_removal_finished (GTD_PROVIDER (user_data),
-                              E_SOURCE (source),
-                              &error);
+  return default_task_list;
 }
 
 static void
-gtd_provider_eds_remove_source_finished (GObject      *source,
-                                         GAsyncResult *result,
-                                         gpointer      user_data)
+gtd_provider_eds_set_default_task_list (GtdProvider *provider,
+                                        GtdTaskList *list)
 {
-  GError *error = NULL;
+  GtdProviderEdsPrivate *priv;
+  ESource *source;
 
-  e_source_remove_finish (E_SOURCE (source),
-                          result,
-                          &error);
+  priv = gtd_provider_eds_get_instance_private (GTD_PROVIDER_EDS (provider));
+  source = g_object_get_data (G_OBJECT (list), "task-list");
 
-  task_list_removal_finished (GTD_PROVIDER (user_data),
-                              E_SOURCE (source),
-                              &error);
+  e_source_registry_set_default_task_list (priv->source_registry, source);
+
+  g_object_notify (G_OBJECT (provider), "default-task-list");
+}
+
+static GtdTask*
+gtd_provider_eds_generate_task (GtdProvider *self)
+{
+  ECalComponent *component;
+  GtdTask *task;
+
+  component = e_cal_component_new ();
+  e_cal_component_set_new_vtype (component, E_CAL_COMPONENT_TODO);
+
+  task = gtd_task_eds_new (component);
+
+  g_object_unref (component);
+
+  return task;
 }
 
 static void
-gtd_provider_eds_commit_source_finished (GObject      *registry,
-                                         GAsyncResult *result,
-                                         gpointer      user_data)
+gtd_provider_iface_init (GtdProviderInterface *iface)
 {
-  GError *error = NULL;
+  iface->get_id = gtd_provider_eds_get_id;
+  iface->get_name = gtd_provider_eds_get_name;
+  iface->get_description = gtd_provider_eds_get_description;
+  iface->get_enabled = gtd_provider_eds_get_enabled;
+  iface->get_icon = gtd_provider_eds_get_icon;
+  iface->create_task = gtd_provider_eds_create_task;
+  iface->update_task = gtd_provider_eds_update_task;
+  iface->remove_task = gtd_provider_eds_remove_task;
+  iface->create_task_list = gtd_provider_eds_create_task_list;
+  iface->update_task_list = gtd_provider_eds_update_task_list;
+  iface->remove_task_list = gtd_provider_eds_remove_task_list;
+  iface->get_task_lists = gtd_provider_eds_get_task_lists;
+  iface->get_default_task_list = gtd_provider_eds_get_default_task_list;
+  iface->set_default_task_list = gtd_provider_eds_set_default_task_list;
+  iface->generate_task = gtd_provider_eds_generate_task;
+}
 
-  g_return_if_fail (GTD_IS_PROVIDER_EDS (user_data));
 
-  gtd_object_set_ready (GTD_OBJECT (user_data), TRUE);
-  e_source_registry_commit_source_finish (E_SOURCE_REGISTRY (registry),
-                                          result,
-                                          &error);
+/*
+ * GObject overrides
+ */
 
-  if (error)
+static void
+gtd_provider_eds_finalize (GObject *object)
+{
+  GtdProviderEds *self = (GtdProviderEds *)object;
+  GtdProviderEdsPrivate *priv = gtd_provider_eds_get_instance_private (self);
+
+  g_signal_handlers_disconnect_by_func (priv->source_registry, default_tasklist_changed_cb, self);
+
+  g_clear_pointer (&priv->clients, g_hash_table_destroy);
+  g_clear_object (&priv->credentials_prompter);
+  g_clear_object (&priv->source_registry);
+
+  G_OBJECT_CLASS (gtd_provider_eds_parent_class)->finalize (object);
+}
+
+static void
+gtd_provider_eds_get_property (GObject    *object,
+                               guint       prop_id,
+                               GValue     *value,
+                               GParamSpec *pspec)
+{
+  GtdProvider *provider = GTD_PROVIDER (object);
+  GtdProviderEdsPrivate *priv = gtd_provider_eds_get_instance_private (GTD_PROVIDER_EDS (object));
+
+
+  switch (prop_id)
     {
-      g_warning ("%s: %s: %s",
-                 G_STRFUNC,
-                 _("Error saving task list"),
-                 error->message);
+    case PROP_DEFAULT_TASKLIST:
+      g_value_set_object (value, gtd_provider_eds_get_default_task_list (provider));
+      break;
 
-      gtd_manager_emit_error_message (gtd_manager_get_default (),
-                                      _("Error saving task list"),
-                                      error->message,
-                                      NULL,
-                                      NULL);
-      g_error_free (error);
-      return;
+    case PROP_DESCRIPTION:
+      g_value_set_string (value, gtd_provider_eds_get_description (provider));
+      break;
+
+    case PROP_ENABLED:
+      g_value_set_boolean (value, gtd_provider_eds_get_enabled (provider));
+      break;
+
+    case PROP_ICON:
+      g_value_set_object (value, gtd_provider_eds_get_icon (provider));
+      break;
+
+    case PROP_ID:
+      g_value_set_string (value, gtd_provider_eds_get_id (provider));
+      break;
+
+    case PROP_NAME:
+      g_value_set_string (value, gtd_provider_eds_get_name (provider));
+      break;
+
+    case PROP_REGISTRY:
+      g_value_set_object (value, priv->source_registry);
+      break;
+
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
     }
 }
 
@@ -818,6 +931,10 @@ gtd_provider_eds_set_property (GObject      *object,
 
   switch (prop_id)
     {
+    case PROP_DEFAULT_TASKLIST:
+      gtd_provider_eds_set_default_task_list (GTD_PROVIDER (self), g_value_get_object (value));
+      break;
+
     case PROP_REGISTRY:
       gtd_provider_eds_set_registry (self, g_value_get_object (value));
       break;
@@ -836,6 +953,13 @@ gtd_provider_eds_class_init (GtdProviderEdsClass *klass)
   object_class->get_property = gtd_provider_eds_get_property;
   object_class->set_property = gtd_provider_eds_set_property;
 
+  g_object_class_override_property (object_class, PROP_DEFAULT_TASKLIST, "default-task-list");
+  g_object_class_override_property (object_class, PROP_DESCRIPTION, "description");
+  g_object_class_override_property (object_class, PROP_ENABLED, "enabled");
+  g_object_class_override_property (object_class, PROP_ICON, "icon");
+  g_object_class_override_property (object_class, PROP_ID, "id");
+  g_object_class_override_property (object_class, PROP_NAME, "name");
+
   g_object_class_install_property (object_class,
                                    PROP_REGISTRY,
                                    g_param_spec_object ("registry",
@@ -879,260 +1003,3 @@ gtd_provider_eds_get_registry (GtdProviderEds *provider)
 
   return priv->source_registry;
 }
-
-void
-gtd_provider_eds_create_task (GtdProviderEds *provider,
-                              GtdTask        *task)
-{
-  GtdProviderEdsPrivate *priv;
-  GtdTaskListEds *tasklist;
-  ECalComponent *component;
-  ECalClient *client;
-  TaskData *data;
-  ESource *source;
-
-  g_return_if_fail (GTD_IS_TASK (task));
-  g_return_if_fail (GTD_IS_TASK_LIST_EDS (gtd_task_get_list (task)));
-
-  priv = gtd_provider_eds_get_instance_private (provider);
-  tasklist = GTD_TASK_LIST_EDS (gtd_task_get_list (task));
-  source = gtd_task_list_eds_get_source (tasklist);
-  client = g_hash_table_lookup (priv->clients, source);
-  component = gtd_task_eds_get_component (GTD_TASK_EDS (task));
-
-  /* Temporary data for async operation */
-  data = task_data_new (provider, (gpointer) task);
-
-  /* The task is not ready until we finish the operation */
-  gtd_object_set_ready (GTD_OBJECT (task), FALSE);
-
-  e_cal_client_create_object (client,
-                              e_cal_component_get_icalcomponent (component),
-                              NULL, // We won't cancel the operation
-                              (GAsyncReadyCallback) gtd_provider_eds_create_task_finished,
-                              data);
-}
-
-void
-gtd_provider_eds_update_task (GtdProviderEds *provider,
-                              GtdTask        *task)
-{
-  GtdProviderEdsPrivate *priv;
-  GtdTaskListEds *tasklist;
-  ECalComponent *component;
-  ECalClient *client;
-  TaskData *data;
-  ESource *source;
-
-  g_return_if_fail (GTD_IS_TASK (task));
-  g_return_if_fail (GTD_IS_TASK_LIST_EDS (gtd_task_get_list (task)));
-
-  priv = gtd_provider_eds_get_instance_private (provider);
-  tasklist = GTD_TASK_LIST_EDS (gtd_task_get_list (task));
-  source = gtd_task_list_eds_get_source (tasklist);
-  client = g_hash_table_lookup (priv->clients, source);
-  component = gtd_task_eds_get_component (GTD_TASK_EDS (task));
-
-  e_cal_component_commit_sequence (component);
-
-  /* Temporary data for async operation */
-  data = task_data_new (provider, (gpointer) task);
-
-  /* The task is not ready until we finish the operation */
-  gtd_object_set_ready (GTD_OBJECT (task), FALSE);
-
-  e_cal_client_modify_object (client,
-                              e_cal_component_get_icalcomponent (component),
-                              E_CAL_OBJ_MOD_THIS,
-                              NULL, // We won't cancel the operation
-                              (GAsyncReadyCallback) gtd_provider_eds_update_task_finished,
-                              data);
-}
-
-void
-gtd_provider_eds_remove_task (GtdProviderEds *provider,
-                              GtdTask        *task)
-{
-
-  GtdProviderEdsPrivate *priv;
-  ECalComponent *component;
-  GtdTaskListEds *tasklist;
-  ECalComponentId *id;
-  ECalClient *client;
-  TaskData *data;
-  ESource *source;
-
-  g_return_if_fail (GTD_IS_TASK (task));
-  g_return_if_fail (GTD_IS_TASK_LIST_EDS (gtd_task_get_list (task)));
-
-  priv = gtd_provider_eds_get_instance_private (provider);
-  tasklist = GTD_TASK_LIST_EDS (gtd_task_get_list (task));
-  source = gtd_task_list_eds_get_source (tasklist);
-  client = g_hash_table_lookup (priv->clients, source);
-  component = gtd_task_eds_get_component (GTD_TASK_EDS (task));
-  id = e_cal_component_get_id (component);
-
-  /* Temporary data for async operation */
-  data = task_data_new (provider, (gpointer) task);
-
-  /* The task is not ready until we finish the operation */
-  gtd_object_set_ready (GTD_OBJECT (task), FALSE);
-
-  e_cal_client_remove_object (client,
-                              id->uid,
-                              id->rid,
-                              E_CAL_OBJ_MOD_THIS,
-                              NULL, // We won't cancel the operation
-                              (GAsyncReadyCallback) gtd_provider_eds_remove_task_finished,
-                              data);
-
-  e_cal_component_free_id (id);
-}
-
-void
-gtd_provider_eds_create_task_list (GtdProviderEds *provider,
-                                   GtdTaskList    *list)
-{
-  GtdProviderEdsPrivate *priv;
-  ESource *source;
-
-  g_return_if_fail (GTD_IS_TASK_LIST_EDS (list));
-  g_return_if_fail (gtd_task_list_eds_get_source (GTD_TASK_LIST_EDS (list)));
-
-  priv = gtd_provider_eds_get_instance_private (provider);
-  source = gtd_task_list_eds_get_source (GTD_TASK_LIST_EDS (list));
-
-  gtd_object_set_ready (GTD_OBJECT (provider), FALSE);
-  e_source_registry_commit_source (priv->source_registry,
-                                   source,
-                                   NULL,
-                                   (GAsyncReadyCallback) gtd_provider_eds_commit_source_finished,
-                                   provider);
-
-  if (e_source_get_remote_creatable (source))
-    {
-      ESource *parent;
-
-      parent = e_source_registry_ref_source (priv->source_registry, e_source_get_parent (source));
-
-      e_source_remote_create (parent,
-                              source,
-                              NULL,
-                              (GAsyncReadyCallback) gtd_provider_eds_remote_create_source_finished,
-                              provider);
-
-      g_object_unref (parent);
-    }
-}
-
-void
-gtd_provider_eds_update_task_list (GtdProviderEds *provider,
-                                   GtdTaskList    *list)
-{
-
-  GtdProviderEdsPrivate *priv;
-  ESource *source;
-
-  g_return_if_fail (GTD_IS_TASK_LIST (list));
-  g_return_if_fail (gtd_task_list_eds_get_source (GTD_TASK_LIST_EDS (list)));
-
-  priv = gtd_provider_eds_get_instance_private (provider);
-  source = gtd_task_list_eds_get_source (GTD_TASK_LIST_EDS (list));
-
-  gtd_object_set_ready (GTD_OBJECT (provider), FALSE);
-  e_source_registry_commit_source (priv->source_registry,
-                                   source,
-                                   NULL,
-                                   (GAsyncReadyCallback) gtd_provider_eds_commit_source_finished,
-                                   provider);
-}
-
-void
-gtd_provider_eds_remove_task_list (GtdProviderEds *provider,
-                                   GtdTaskList    *list)
-{
-  ESource *source;
-
-  g_return_if_fail (GTD_IS_TASK_LIST (list));
-  g_return_if_fail (gtd_task_list_eds_get_source (GTD_TASK_LIST_EDS (list)));
-
-  source = gtd_task_list_eds_get_source (GTD_TASK_LIST_EDS (list));
-
-  gtd_object_set_ready (GTD_OBJECT (provider), FALSE);
-
-  if (e_source_get_remote_deletable (source))
-    {
-      e_source_remote_delete (source,
-                              NULL,
-                              (GAsyncReadyCallback) gtd_provider_eds_remote_delete_finished,
-                              provider);
-    }
-  else
-    {
-      e_source_remove (source,
-                       NULL,
-                       (GAsyncReadyCallback) gtd_provider_eds_remove_source_finished,
-                       provider);
-    }
-}
-
-GList*
-gtd_provider_eds_get_task_lists (GtdProviderEds *provider)
-{
-  GtdProviderEdsPrivate *priv = gtd_provider_eds_get_instance_private (provider);
-
-  return priv->task_lists;
-}
-
-GtdTaskList*
-gtd_provider_eds_get_default_task_list (GtdProviderEds *provider)
-{
-  GtdProviderEdsPrivate *priv;
-  GtdTaskList *default_task_list;
-  ESource *default_source;
-
-  priv = gtd_provider_eds_get_instance_private (provider);
-  default_source = e_source_registry_ref_default_task_list (priv->source_registry);
-  default_task_list = g_object_get_data (G_OBJECT (default_source), "task-list");
-
-  g_clear_object (&default_source);
-
-  if (default_task_list &&
-      gtd_task_list_get_provider (default_task_list) != GTD_PROVIDER (provider))
-    {
-      return NULL;
-    }
-
-  return default_task_list;
-}
-
-void
-gtd_provider_eds_set_default_task_list (GtdProviderEds *provider,
-                                        GtdTaskList    *list)
-{
-  GtdProviderEdsPrivate *priv;
-  ESource *source;
-
-  priv = gtd_provider_eds_get_instance_private (provider);
-  source = g_object_get_data (G_OBJECT (list), "task-list");
-
-  e_source_registry_set_default_task_list (priv->source_registry, source);
-
-  g_object_notify (G_OBJECT (provider), "default-task-list");
-}
-
-GtdTask*
-gtd_provider_eds_generate_task (GtdProviderEds *self)
-{
-  ECalComponent *component;
-  GtdTask *task;
-
-  component = e_cal_component_new ();
-  e_cal_component_set_new_vtype (component, E_CAL_COMPONENT_TODO);
-
-  task = gtd_task_eds_new (component);
-
-  g_object_unref (component);
-
-  return task;
-}
