@@ -24,7 +24,13 @@ gi.require_version('Peas', '1.0')
 
 from gi.repository import Gio, GLib, GObject, Gtd, Gtk, Peas
 
-from gettext import gettext as _
+try:
+    import gettext
+    gettext.bindtextdomain('gnome-todo')
+    gettext.textdomain('gnome-todo')
+    _ = gettext.gettext
+except:
+    _ = lambda s: s
 
 
 class UnscheduledPanel(Gtk.Box, Gtd.Panel):
@@ -42,14 +48,15 @@ class UnscheduledPanel(Gtk.Box, Gtd.Panel):
         manager.connect('list-removed', self._count_tasks)
 
         self.task_counter = 0
-        self.tasklist = Gtd.TaskList()
 
         self.view = Gtd.TaskListView(hexpand=True,
                                      vexpand=True)
         self.view.set_show_list_name(True)
-        self.view.set_task_list(self.tasklist)
         self.view.set_handle_subtasks(False)
 
+        self.menu = Gio.Menu()
+        self.menu.append(_("Clear completed tasks…"), "list.clear-completed-tasks")
+
         self.add(self.view)
         self.show_all()
 
@@ -61,15 +68,21 @@ class UnscheduledPanel(Gtk.Box, Gtd.Panel):
         self.task_counter = 0
 
         manager = Gtd.Manager.get_default()
-        current_tasks = self.tasklist.get_tasks()
+        current_tasks = []
 
         for tasklist in manager.get_task_lists():
             for task in tasklist.get_tasks():
-                if task.get_due_date() is None:
-                    if not task in current_tasks:
-                        self.tasklist.save_task(task)
-                    if not task.get_complete():
-                        self.task_counter += 1
+
+                if not task.get_due_date() is None:
+                    continue
+
+                current_tasks.append(task)
+
+                # Update the counter
+                if not task.get_complete():
+                    self.task_counter += 1
+
+        self.view.set_list(current_tasks)
 
         if previous_task_counter != self.task_counter:
             self.notify("title")
@@ -78,7 +91,7 @@ class UnscheduledPanel(Gtk.Box, Gtd.Panel):
         return None
 
     def do_get_menu(self):
-        return None
+        return self.menu
 
     def do_get_panel_name(self):
         return "unscheduled-panel"
