@@ -16,6 +16,8 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
+#define G_LOG_DOMAIN "GtdTaskList"
+
 #include "interfaces/gtd-provider.h"
 #include "gtd-task.h"
 #include "gtd-task-list.h"
@@ -209,9 +211,9 @@ gtd_task_list_class_init (GtdTaskListClass *klass)
                              G_PARAM_READWRITE));
 
   /**
-   * GtdTaskList::source:
+   * GtdTaskList::provider:
    *
-   * The parent source of the list.
+   * The data provider of the list.
    */
   g_object_class_install_property (
         object_class,
