@@ -16,6 +16,8 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
+#define G_LOG_DOMAIN "GtdTask"
+
 #include "gtd-task.h"
 #include "gtd-task-list.h"
 
@@ -46,7 +48,7 @@ typedef struct
   gchar           *title;
 
   gint32           priority;
-  gboolean         complete : 1;
+  gboolean         complete;
 } GtdTaskPrivate;
 
 G_DEFINE_TYPE_WITH_PRIVATE (GtdTask, gtd_task, GTD_TYPE_OBJECT)
