@@ -16,6 +16,8 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
+#define G_LOG_DOMAIN "GtdProviderPopover"
+
 #include "gtd-manager.h"
 #include "interfaces/gtd-provider.h"
 #include "gtd-provider-popover.h"
@@ -91,7 +93,7 @@ create_task_list (GtdProviderPopover *popover)
   task_list = gtd_task_list_new (provider);
   gtd_task_list_set_name (task_list, name);
 
-  gtd_provider_create_task_list (provider, task_list);
+  gtd_manager_create_task_list (gtd_manager_get_default (), task_list);
 }
 
 static void
